@@ -55,9 +55,11 @@
  */
 typedef struct {
 #if defined(INFIX_OS_WINDOWS)
-    HANDLE handle; /**< The handle from `VirtualAlloc`, needed for `VirtualFree`. */
+    HANDLE handle;           /**< The handle from `VirtualAlloc`, needed for `VirtualFree`. */
+    void * seh_registration; /**< (Windows x64) Opaque handle from `RtlAddFunctionTable`. */
 #else
-    int shm_fd; /**< The file descriptor for shared memory on dual-mapping POSIX systems. -1 otherwise. */
+    int shm_fd;          /**< The file descriptor for shared memory on dual-mapping POSIX systems. -1 otherwise. */
+    void * eh_frame_ptr; /**< (POSIX) Pointer to the registered .eh_frame data. */
 #endif
     void * rx_ptr; /**< The read-execute memory address. This is the callable function pointer. */
     void * rw_ptr; /**< The read-write memory address. The JIT compiler writes machine code here. */
@@ -96,6 +98,9 @@ struct infix_forward_t {
     size_t num_fixed_args;     /**< The number of non-variadic arguments. */
     void * target_fn;          /**< The target C function pointer (for bound trampolines), or `nullptr` for unbound. */
     bool is_direct_trampoline; /**< If true, this is a high-performance direct marshalling trampoline. */
+    bool is_safe;              /**< If true, the trampoline wraps the call in an exception handler. */
+    size_t ref_count;          /**< Reference count for deduplication and shared ownership. */
+    char * signature;          /**< The normalized signature string used to create this trampoline. */
 };
 /**
  * @brief A function pointer to the universal C dispatcher for reverse calls.
@@ -143,6 +148,23 @@ struct infix_arena_t {
     struct infix_arena_t * next_block; /**< A pointer to the next block in the chain, if this one is full. */
     size_t block_size;                 /**< The size of this specific block's buffer, for chained arenas. */
 };
+// Mutex Abstraction for Internal Synchronization
+#if defined(INFIX_OS_WINDOWS)
+#include <windows.h>
+typedef SRWLOCK infix_mutex_t;
+#define INFIX_MUTEX_INITIALIZER SRWLOCK_INIT
+#define INFIX_MUTEX_LOCK(m) AcquireSRWLockExclusive(m)
+#define INFIX_MUTEX_UNLOCK(m) ReleaseSRWLockExclusive(m)
+#define INFIX_MUTEX_DESTROY(m) ((void)0)
+#else
+#include <pthread.h>
+typedef pthread_mutex_t infix_mutex_t;
+#define INFIX_MUTEX_INITIALIZER PTHREAD_MUTEX_INITIALIZER
+#define INFIX_MUTEX_LOCK(m) pthread_mutex_lock(m)
+#define INFIX_MUTEX_UNLOCK(m) pthread_mutex_unlock(m)
+#define INFIX_MUTEX_DESTROY(m) pthread_mutex_destroy(m)
+#endif
+
 /**
  * @struct _infix_registry_entry_t
  * @brief A single entry in the registry's hash table.
@@ -151,6 +173,7 @@ struct infix_arena_t {
  */
 typedef struct _infix_registry_entry_t {
     const char * name;                     /**< The registered name of the type. */
+    uint64_t hash;                         /**< The pre-calculated djb2 hash of the name. */
     infix_type * type;                     /**< A pointer to the canonical `infix_type` object. */
     bool is_forward_declaration;           /**< `true` if this is just a forward declaration (`@Name;`). */
     struct _infix_registry_entry_t * next; /**< The next entry in the hash bucket chain. */
@@ -159,35 +182,30 @@ typedef struct _infix_registry_entry_t {
  * @struct infix_registry_t
  * @brief Internal definition of a named type registry.
  * @details Implemented as a hash table with separate chaining for collision resolution.
- * All memory for the table, its entries, and the canonical `infix_type` objects
- * it stores are owned by a single arena for simple lifecycle management.
+ * The canonical `infix_type` objects and entry metadata are owned by a single arena,
+ * while the internal bucket array is heap-allocated to allow for efficient resizing.
  */
 struct infix_registry_t {
     infix_arena_t * arena;              /**< The arena that owns all type metadata and entry structs. */
     bool is_external_arena;             /**< True if the arena is user-provided and should not be freed. */
     size_t num_buckets;                 /**< The number of buckets in the hash table. */
     size_t num_items;                   /**< The total number of items in the registry. */
-    _infix_registry_entry_t ** buckets; /**< The array of hash table buckets (linked list heads). */
+    _infix_registry_entry_t ** buckets; /**< The array of hash table buckets (heap-allocated). */
 };
 /**
- * @struct infix_registry_iterator_t
- * @brief Internal definition of a registry iterator.
- * @details This struct holds the complete state needed to traverse the registry's
- * internal hash table, including the current bucket and the current entry within
- * that bucket's linked list.
+ * @struct parser_state
+ * @brief Holds the complete state of the recursive descent parser during a single parse operation.
  */
-struct infix_registry_iterator_t {
-    const infix_registry_t * registry;             /**< The registry being iterated. */
-    size_t current_bucket;                         /**< The index of the current hash bucket. */
-    const _infix_registry_entry_t * current_entry; /**< The current entry in the bucket's chain. */
-};
+typedef struct {
+    const char * p;        /**< The current read position (cursor) in the signature string. */
+    const char * start;    /**< The beginning of the signature string, used for calculating error positions. */
+    infix_arena_t * arena; /**< The temporary arena for allocating the raw, unresolved type graph. */
+    int depth;             /**< The current recursion depth, to prevent stack overflows. */
+} parser_state;
+
 /**
  * @struct code_buffer
  * @brief A dynamic buffer for staged machine code generation.
- * @details This structure is used during the JIT compilation process. ABI-specific
- * emitters append instruction bytes to this buffer. It automatically grows as needed,
- * allocating memory from a temporary arena that is destroyed after the final
- * code is copied to executable memory.
  */
 typedef struct {
     uint8_t * code;        /**< A pointer to the code buffer, allocated from the arena. */
@@ -292,6 +310,9 @@ typedef struct {
     size_t num_stack_args;       /**< The number of arguments passed on the stack. */
     size_t num_args;             /**< The total number of arguments. */
     void * target_fn;            /**< The target function address. */
+    uint32_t max_align;          /**< Maximum required alignment for any argument or the stack. */
+    uint32_t prologue_size;      /**< Size of the generated prologue in bytes. */
+    uint32_t epilogue_offset;    /**< Offset from the start of the JIT block to the epilogue. */
 } infix_call_frame_layout;
 /**
  * @struct infix_reverse_call_frame_layout
@@ -307,6 +328,8 @@ typedef struct {
     int32_t saved_args_offset;    /**< Stack offset for the area where argument data is stored/marshalled. */
     int32_t gpr_save_area_offset; /**< (Win x64) Stack offset for saving non-volatile GPRs. */
     int32_t xmm_save_area_offset; /**< (Win x64) Stack offset for saving non-volatile XMMs. */
+    uint32_t max_align;           /**< Maximum required alignment for any argument or the stack. */
+    uint32_t prologue_size;       /**< Size of the generated prologue in bytes. */
 } infix_reverse_call_frame_layout;
 /**
  * @brief Defines the ABI-specific implementation interface for forward trampolines.
@@ -467,6 +490,8 @@ typedef struct {
     void * target_fn;                ///< The target C function address.
     bool return_value_in_memory;     ///< `true` if the return value uses a hidden pointer argument.
     infix_direct_arg_layout * args;  ///< An array of layout info for each argument.
+    uint32_t prologue_size;          ///< Size of the generated prologue in bytes.
+    uint32_t epilogue_offset;        ///< Offset from the start of the JIT block to the epilogue.
 } infix_direct_call_frame_layout;
 
 /**
@@ -509,7 +534,7 @@ typedef struct {
  * @param code The specific error code.
  * @param position For parser errors, the byte offset into the signature string where the error occurred.
  */
-void _infix_set_error(infix_error_category_t category, infix_error_code_t code, size_t position);
+INFIX_INTERNAL void _infix_set_error(infix_error_category_t category, infix_error_code_t code, size_t position);
 /**
  * @brief Sets the thread-local error state for a system-level error.
  * @details Located in `src/core/error.c`, this is used for errors originating from
@@ -519,16 +544,21 @@ void _infix_set_error(infix_error_category_t category, infix_error_code_t code,
  * @param system_code The OS-specific error code (e.g., from `errno` or `GetLastError`).
  * @param msg An optional custom message from the OS (e.g., from `dlerror`).
  */
-void _infix_set_system_error(infix_error_category_t category,
-                             infix_error_code_t code,
-                             long system_code,
-                             const char * msg);
+INFIX_INTERNAL void _infix_set_system_error(infix_error_category_t category,
+                                            infix_error_code_t code,
+                                            long system_code,
+                                            const char * msg);
 /**
  * @brief Clears the thread-local error state.
  * @details Located in `src/core/error.c`. This is called at the beginning of every public
  * API function to ensure that a prior error from an unrelated call is not accidentally returned.
  */
-void _infix_clear_error(void);
+INFIX_INTERNAL void _infix_clear_error(void);
+INFIX_INTERNAL void skip_whitespace(parser_state * state);
+INFIX_INTERNAL void _infix_set_parser_error(parser_state * state, infix_error_code_t code);
+INFIX_INTERNAL infix_type * parse_type(parser_state * state);
+INFIX_INTERNAL infix_type * parse_primitive(parser_state * state);
+
 /**
  * @brief Recalculates the layout of a fully resolved type graph.
  * @details Located in `src/core/types.c`. This is the "Layout" stage of the data pipeline.
@@ -537,7 +567,7 @@ void _infix_clear_error(void);
  * resolved graph.
  * @param[in,out] type The root of the type graph to recalculate. The graph is modified in-place.
  */
-void _infix_type_recalculate_layout(infix_type * type);
+INFIX_INTERNAL void _infix_type_recalculate_layout(infix_type * type);
 /**
  * @brief Resolves all named type references in a type graph in-place.
  * @details Located in `src/core/type_registry.c`. This is the "Resolve" stage of the
@@ -547,7 +577,8 @@ void _infix_type_recalculate_layout(infix_type * type);
  * @param[in] registry The registry to use for lookups.
  * @return `INFIX_SUCCESS` on success, or an error if a name cannot be resolved.
  */
-c23_nodiscard infix_status _infix_resolve_type_graph_inplace(infix_type ** type_ptr, infix_registry_t * registry);
+INFIX_INTERNAL c23_nodiscard infix_status _infix_resolve_type_graph_inplace(infix_type ** type_ptr,
+                                                                            infix_registry_t * registry);
 /**
  * @brief The internal core of the signature parser.
  * @details Located in `src/core/signature.c`. This is the "Parse" stage of the data pipeline.
@@ -558,7 +589,7 @@ c23_nodiscard infix_status _infix_resolve_type_graph_inplace(infix_type ** type_
  * @param[in] signature The signature string to parse.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status _infix_parse_type_internal(infix_type **, infix_arena_t **, const char *);
+INFIX_INTERNAL c23_nodiscard infix_status _infix_parse_type_internal(infix_type **, infix_arena_t **, const char *);
 /**
  * @brief An internal-only function to serialize a type's body without its registered name.
  * @details Located in `src/core/signature.c`. Unlike `infix_type_print`, which will
@@ -570,7 +601,10 @@ c23_nodiscard infix_status _infix_parse_type_internal(infix_type **, infix_arena
  * @param[in] dialect The output format dialect.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status _infix_type_print_body_only(char *, size_t, const infix_type *, infix_print_dialect_t);
+INFIX_INTERNAL c23_nodiscard infix_status _infix_type_print_body_only(char *,
+                                                                      size_t,
+                                                                      const infix_type *,
+                                                                      infix_print_dialect_t);
 /**
  * @brief Performs a deep copy of a type graph into a destination arena.
  * @details Located in `src/core/types.c`. This is the "Copy" stage of the data pipeline,
@@ -580,7 +614,7 @@ c23_nodiscard infix_status _infix_type_print_body_only(char *, size_t, const inf
  * @param[in] src_type The source type graph to copy.
  * @return A pointer to the newly created copy in `dest_arena`, or `nullptr` on failure.
  */
-infix_type * _copy_type_graph_to_arena(infix_arena_t *, const infix_type *);
+INFIX_INTERNAL infix_type * _copy_type_graph_to_arena(infix_arena_t *, const infix_type *);
 /**
  * @brief Estimates the total memory required to deep-copy a complete type graph.
  * @details Located in `src/core/types.c`. This function recursively walks the entire
@@ -590,7 +624,7 @@ infix_type * _copy_type_graph_to_arena(infix_arena_t *, const infix_type *);
  * @param[in] type The root of the type graph to estimate.
  * @return The estimated size in bytes required for a deep copy.
  */
-size_t _infix_estimate_graph_size(infix_arena_t * temp_arena, const infix_type * type);
+INFIX_INTERNAL size_t _infix_estimate_graph_size(infix_arena_t * temp_arena, const infix_type * type);
 /**
  * @brief Gets the ABI v-table for forward calls for the current platform.
  * @details See `src/jit/trampoline.c`. This function is the entry point to the ABI
@@ -598,19 +632,19 @@ size_t _infix_estimate_graph_size(infix_arena_t * temp_arena, const infix_type *
  * compile-time ABI detection.
  * @return A pointer to the active `infix_forward_abi_spec`.
  */
-const infix_forward_abi_spec * get_current_forward_abi_spec(void);
+INFIX_INTERNAL const infix_forward_abi_spec * get_current_forward_abi_spec(void);
 /**
  * @brief Gets the ABI v-table for reverse calls for the current platform.
  * @details See `src/jit/trampoline.c`. This function mirrors `get_current_forward_abi_spec`
  * for reverse call trampolines.
  * @return A pointer to the active `infix_reverse_abi_spec`.
  */
-const infix_reverse_abi_spec * get_current_reverse_abi_spec(void);
+INFIX_INTERNAL const infix_reverse_abi_spec * get_current_reverse_abi_spec(void);
 /**
  * @brief Gets the ABI v-table for direct marshalling forward calls for the current platform.
  * @return A pointer to the active `infix_direct_forward_abi_spec`.
  */
-const infix_direct_forward_abi_spec * get_current_direct_forward_abi_spec(void);
+INFIX_INTERNAL const infix_direct_forward_abi_spec * get_current_direct_forward_abi_spec(void);
 /**
  * @brief Initializes a code buffer for JIT code generation.
  * @details See `src/jit/trampoline.c`. Associates the buffer with a temporary
@@ -618,7 +652,7 @@ const infix_direct_forward_abi_spec * get_current_direct_forward_abi_spec(void);
  * @param[out] buf A pointer to the `code_buffer` to initialize.
  * @param[in] arena The temporary arena to use for the buffer's memory.
  */
-void code_buffer_init(code_buffer * buf, infix_arena_t * arena);
+INFIX_INTERNAL void code_buffer_init(code_buffer * buf, infix_arena_t * arena);
 /**
  * @brief Appends raw bytes to a code buffer, reallocating within its arena if necessary.
  * @details See `src/jit/trampoline.c`. This is the fundamental operation for building
@@ -627,34 +661,25 @@ void code_buffer_init(code_buffer * buf, infix_arena_t * arena);
  * @param[in] data A pointer to the bytes to append.
  * @param[in] len The number of bytes to append.
  */
-void code_buffer_append(code_buffer * buf, const void * data, size_t len);
+INFIX_INTERNAL void code_buffer_append(code_buffer * buf, const void * data, size_t len);
 /**
  * @brief A convenience wrapper to append a single byte to a code buffer.
  * @param[in,out] buf The code buffer.
  * @param[in] byte The byte to append.
  */
-void emit_byte(code_buffer * buf, uint8_t byte);
+INFIX_INTERNAL void emit_byte(code_buffer * buf, uint8_t byte);
 /**
  * @brief A convenience wrapper to append a 32-bit integer (little-endian) to a code buffer.
  * @param[in,out] buf The code buffer.
  * @param[in] value The 32-bit integer to append.
  */
-void emit_int32(code_buffer * buf, int32_t value);
+INFIX_INTERNAL void emit_int32(code_buffer * buf, int32_t value);
 /**
  * @brief A convenience wrapper to append a 64-bit integer (little-endian) to a code buffer.
  * @param[in,out] buf The code buffer.
  * @param[in] value The 64-bit integer to append.
  */
-void emit_int64(code_buffer * buf, int64_t value);
-/**
- * @brief The core implementation for creating all forward trampolines.
- * @details Located in `src/jit/trampoline.c`, this function orchestrates the
- * entire JIT pipeline for forward calls, from ABI classification to code generation
- * and final handle creation.
- * @return `INFIX_SUCCESS` on success.
- */
-c23_nodiscard infix_status
-_infix_forward_create_internal(infix_forward_t **, infix_type *, infix_type **, size_t, size_t, void *);
+INFIX_INTERNAL void emit_int64(code_buffer * buf, int64_t value);
 /**
  * @brief Allocates a block of executable memory using the platform's W^X strategy.
  * @details Located in `src/jit/executor.c`. This is a platform-specific function
@@ -662,23 +687,35 @@ _infix_forward_create_internal(infix_forward_t **, infix_type *, infix_type **,
  * @param size The number of bytes to allocate.
  * @return An `infix_executable_t` handle containing pointers to the allocated memory.
  */
-c23_nodiscard infix_executable_t infix_executable_alloc(size_t size);
+INFIX_INTERNAL c23_nodiscard infix_executable_t infix_executable_alloc(size_t size);
 /**
  * @brief Frees a block of executable memory and applies guard pages to prevent use-after-free.
  * @details Located in `src/jit/executor.c`. Before freeing, it attempts to change
  * the memory's protection to be inaccessible, causing an immediate crash on a UAF.
  * @param exec The handle to the memory block to free.
  */
-void infix_executable_free(infix_executable_t exec);
+INFIX_INTERNAL void infix_executable_free(infix_executable_t exec);
+typedef enum {
+    INFIX_EXECUTABLE_FORWARD,
+    INFIX_EXECUTABLE_SAFE_FORWARD,
+    INFIX_EXECUTABLE_REVERSE,
+    INFIX_EXECUTABLE_DIRECT
+} infix_executable_category_t;
+
 /**
  * @brief Makes a block of JIT memory executable, completing the W^X process.
  * @details Located in `src/jit/executor.c`. For single-map platforms, this calls
  * `VirtualProtect` or `mprotect`. For dual-map platforms, this is a no-op. It
  * also handles instruction cache flushing on relevant architectures like AArch64.
  * @param exec The handle to the memory block to make executable.
+ * @param prologue_size The size of the generated prologue in bytes.
+ * @param category The type of trampoline being finalized.
  * @return `true` on success, `false` on failure.
  */
-c23_nodiscard bool infix_executable_make_executable(infix_executable_t exec);
+INFIX_INTERNAL c23_nodiscard bool infix_executable_make_executable(infix_executable_t * exec,
+                                                                   infix_executable_category_t category,
+                                                                   uint32_t prologue_size,
+                                                                   uint32_t epilogue_offset);
 /**
  * @brief Allocates a block of standard memory for later protection.
  * @details Located in `src/jit/executor.c`. This is used to allocate the memory
@@ -686,13 +723,13 @@ c23_nodiscard bool infix_executable_make_executable(infix_executable_t exec);
  * @param size The number of bytes to allocate.
  * @return An `infix_protected_t` handle.
  */
-c23_nodiscard infix_protected_t infix_protected_alloc(size_t size);
+INFIX_INTERNAL c23_nodiscard infix_protected_t infix_protected_alloc(size_t size);
 /**
  * @brief Frees a block of protected memory.
  * @details Located in `src/jit/executor.c`.
  * @param prot The memory block to free.
  */
-void infix_protected_free(infix_protected_t prot);
+INFIX_INTERNAL void infix_protected_free(infix_protected_t prot);
 /**
  * @brief Makes a block of memory read-only for security hardening.
  * @details Located in `src/jit/executor.c`. This is called on the `infix_reverse_t`
@@ -700,7 +737,7 @@ void infix_protected_free(infix_protected_t prot);
  * @param prot The memory block to make read-only.
  * @return `true` on success, `false` on failure.
  */
-c23_nodiscard bool infix_protected_make_readonly(infix_protected_t prot);
+INFIX_INTERNAL c23_nodiscard bool infix_protected_make_readonly(infix_protected_t prot);
 /**
  * @brief The universal C entry point for all reverse call trampolines.
  * @details Located in `src/jit/executor.c`, this function is called by the JIT-compiled
@@ -710,7 +747,10 @@ c23_nodiscard bool infix_protected_make_readonly(infix_protected_t prot);
  * @param[out] return_value_ptr A pointer to the stack buffer for the return value.
  * @param[in] args_array A pointer to the `void**` array of argument pointers.
  */
-void infix_internal_dispatch_callback_fn_impl(infix_reverse_t * context, void * return_value_ptr, void ** args_array);
+INFIX_INTERNAL void infix_internal_dispatch_callback_fn_impl(infix_reverse_t * context,
+                                                             void * return_value_ptr,
+                                                             void ** args_array);
+
 // Utility Macros & Inlines
 /** @brief Appends a sequence of bytes (e.g., an instruction opcode) to a code buffer. */
 #define EMIT_BYTES(buf, ...)                             \
@@ -728,7 +768,15 @@ static inline size_t _infix_align_up(size_t value, size_t alignment) {
     return (value + alignment - 1) & ~(alignment - 1);
 }
 /**
- * @brief A fast inline check to determine if an `infix_type` is a `float`.
+ * @brief A fast inline check to determine if an `infix_type` is a half-precision float (`float16`).
+ * @param type The type to check.
+ * @return `true` if the type is a float16 primitive.
+ */
+static inline bool is_float16(const infix_type * type) {
+    return type->category == INFIX_TYPE_PRIMITIVE && type->meta.primitive_id == INFIX_PRIMITIVE_FLOAT16;
+}
+/**
+ * @brief A fast inline check to determine if an `infix_type` is a `float` (32-bit).
  * @param type The type to check.
  * @return `true` if the type is a float primitive.
  */
@@ -757,4 +805,15 @@ static inline bool is_long_double(const infix_type * type) {
 #elif defined(INFIX_ABI_AAPCS64)
 #include "arch/aarch64/abi_arm64_emitters.h"
 #endif
+
+// Trampoline Caching
+INFIX_INTERNAL infix_forward_t * _infix_cache_lookup(const char * signature, void * target_fn, bool is_safe);
+INFIX_INTERNAL void _infix_cache_insert(infix_forward_t * trampoline);
+INFIX_INTERNAL bool _infix_cache_remove(infix_forward_t * trampoline);
+INFIX_INTERNAL void _infix_cache_release(infix_forward_t * trampoline);
+INFIX_INTERNAL void _infix_cache_clear(void);
+
+// Internal Cleanup
+INFIX_INTERNAL void _infix_forward_destroy_internal(infix_forward_t * trampoline);
+
 /** @endinternal */
