@@ -40,6 +40,9 @@
 #include <stdlib.h>
 #include <string.h>
 // Static Descriptors for Primitive and Built-in Types
+
+#define INFIX_MAX_ALIGNMENT (1024 * 1024)  // 1MB Safety Limit
+
 /**
  * @internal
  * @def INFIX_TYPE_INIT
@@ -112,12 +115,20 @@ static infix_type _infix_type_uint128 = INFIX_TYPE_INIT(INFIX_PRIMITIVE_UINT128,
 /** @internal Static singleton for the `__int128_t` primitive type (GCC/Clang only). */
 static infix_type _infix_type_sint128 = INFIX_TYPE_INIT(INFIX_PRIMITIVE_SINT128, __int128_t);
 #endif
+/** @internal Static singleton for the `_Float16` primitive type. */
+static infix_type _infix_type_float16 = {.name = nullptr,
+                                         .category = INFIX_TYPE_PRIMITIVE,
+                                         .size = 2,
+                                         .alignment = 2,
+                                         .is_arena_allocated = false,
+                                         .arena = nullptr,
+                                         .source_offset = 0,
+                                         .meta.primitive_id = INFIX_PRIMITIVE_FLOAT16};
 /** @internal Static singleton for the `float` primitive type. */
 static infix_type _infix_type_float = INFIX_TYPE_INIT(INFIX_PRIMITIVE_FLOAT, float);
 /** @internal Static singleton for the `double` primitive type. */
 static infix_type _infix_type_double = INFIX_TYPE_INIT(INFIX_PRIMITIVE_DOUBLE, double);
-#if defined(INFIX_COMPILER_MSVC) || (defined(INFIX_OS_WINDOWS) && defined(INFIX_COMPILER_CLANG)) || \
-    defined(INFIX_OS_MACOS)
+#if defined(INFIX_COMPILER_MSVC) || (defined(INFIX_OS_WINDOWS) && defined(INFIX_COMPILER_CLANG))
 // On these platforms, long double is just an alias for double, so no separate singleton is needed.
 #else
 /** @internal Static singleton for the `long double` primitive type (where it is distinct from `double`). */
@@ -130,7 +141,7 @@ static infix_type _infix_type_long_double = INFIX_TYPE_INIT(INFIX_PRIMITIVE_LONG
  * @return A pointer to the corresponding static `infix_type` singleton descriptor. This pointer does not need to be
  * freed.
  */
-c23_nodiscard infix_type * infix_type_create_primitive(infix_primitive_type_id id) {
+INFIX_API c23_nodiscard infix_type * infix_type_create_primitive(infix_primitive_type_id id) {
     switch (id) {
     case INFIX_PRIMITIVE_BOOL:
         return &_infix_type_bool;
@@ -156,17 +167,20 @@ c23_nodiscard infix_type * infix_type_create_primitive(infix_primitive_type_id i
     case INFIX_PRIMITIVE_SINT128:
         return &_infix_type_sint128;
 #endif
+    case INFIX_PRIMITIVE_FLOAT16:
+        return &_infix_type_float16;
     case INFIX_PRIMITIVE_FLOAT:
         return &_infix_type_float;
     case INFIX_PRIMITIVE_DOUBLE:
         return &_infix_type_double;
     case INFIX_PRIMITIVE_LONG_DOUBLE:
-#if defined(INFIX_COMPILER_MSVC) || (defined(INFIX_OS_WINDOWS) && defined(INFIX_COMPILER_CLANG)) || \
-    defined(INFIX_OS_MACOS)
-        // On platforms where long double is just an alias for double, return the double singleton
-        // to maintain consistent type representation.
+#if defined(INFIX_COMPILER_MSVC) || (defined(INFIX_OS_WINDOWS) && defined(INFIX_COMPILER_CLANG))
+        // On MSVC and macOS/Clang (sometimes), long double is an alias for double.
+        // We map to the double singleton to maintain type identity.
         return &_infix_type_double;
 #else
+        // On MinGW and Linux, long double is distinct (16 bytes).
+        // We MUST use the distinct type to handle layout and passing correctly.
         return &_infix_type_long_double;
 #endif
     default:
@@ -178,12 +192,12 @@ c23_nodiscard infix_type * infix_type_create_primitive(infix_primitive_type_id i
  * @brief Creates a static descriptor for a generic pointer (`void*`).
  * @return A pointer to the static `infix_type` descriptor. Does not need to be freed.
  */
-c23_nodiscard infix_type * infix_type_create_pointer(void) { return &_infix_type_pointer; }
+INFIX_API c23_nodiscard infix_type * infix_type_create_pointer(void) { return &_infix_type_pointer; }
 /**
  * @brief Creates a static descriptor for the `void` type.
  * @return A pointer to the static `infix_type` descriptor. Does not need to be freed.
  */
-c23_nodiscard infix_type * infix_type_create_void(void) { return &_infix_type_void; }
+INFIX_API c23_nodiscard infix_type * infix_type_create_void(void) { return &_infix_type_void; }
 /**
  * @brief A factory function to create an `infix_struct_member`.
  * @param[in] name The name of the member (optional, can be `nullptr`).
@@ -191,7 +205,7 @@ c23_nodiscard infix_type * infix_type_create_void(void) { return &_infix_type_vo
  * @param[in] offset The byte offset of the member from the start of its parent aggregate.
  * @return An initialized `infix_struct_member` object.
  */
-infix_struct_member infix_type_create_member(const char * name, infix_type * type, size_t offset) {
+INFIX_API infix_struct_member infix_type_create_member(const char * name, infix_type * type, size_t offset) {
     return (infix_struct_member){name, type, offset, 0, 0, false};
 }
 /**
@@ -202,10 +216,10 @@ infix_struct_member infix_type_create_member(const char * name, infix_type * typ
  * @param[in] bit_width The width in bits.
  * @return An initialized `infix_struct_member` object.
  */
-infix_struct_member infix_type_create_bitfield_member(const char * name,
-                                                      infix_type * type,
-                                                      size_t offset,
-                                                      uint8_t bit_width) {
+INFIX_API infix_struct_member infix_type_create_bitfield_member(const char * name,
+                                                                infix_type * type,
+                                                                size_t offset,
+                                                                uint8_t bit_width) {
     return (infix_struct_member){name, type, offset, bit_width, 0, true};
 }
 
@@ -216,58 +230,61 @@ infix_struct_member infix_type_create_bitfield_member(const char * name,
  */
 static bool _layout_struct(infix_type * type) {
     size_t current_byte_offset = 0;
-    uint8_t current_bit_offset = 0;  // 0-7 bits used in the current byte
+    size_t current_unit_offset = 0;
+    size_t current_unit_size = 0;
+    uint32_t current_unit_bits_used = 0;
     size_t max_alignment = 1;
+    bool in_bitfield = false;
 
     for (size_t i = 0; i < type->meta.aggregate_info.num_members; ++i) {
         infix_struct_member * member = &type->meta.aggregate_info.members[i];
+        infix_type * mtype = member->type;
 
-        // 1. Handle Flexible Array Members (FAM)
-        if (member->type->category == INFIX_TYPE_ARRAY && member->type->meta.array_info.is_flexible) {
-            // Flush any pending bits to the next byte
-            if (current_bit_offset > 0) {
-                if (current_byte_offset == SIZE_MAX) {
+        if (member->is_bitfield) {
+            size_t align = mtype->alignment;
+            if (align == 0)
+                align = 1;
+            if (align > max_alignment && !type->meta.aggregate_info.is_packed)
+                max_alignment = align;
+
+            // Zero-width bitfield or type mismatch or doesn't fit -> start new unit
+            if (member->bit_width == 0 || !in_bitfield || mtype->size != current_unit_size ||
+                current_unit_bits_used + member->bit_width > mtype->size * 8) {
+
+                // Align to start of new unit
+                size_t start = _infix_align_up(current_byte_offset, align);
+                if (start < current_byte_offset) {
                     _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
                     return false;
                 }
-                current_byte_offset++;
-                current_bit_offset = 0;
+
+                current_unit_offset = start;
+                current_unit_size = mtype->size;
+                current_unit_bits_used = 0;
+                in_bitfield = true;
+
+                current_byte_offset = current_unit_offset;
             }
 
-            // FAM aligns according to its element type.
-            size_t member_align = member->type->alignment;
-            if (member_align == 0)
-                member_align = 1;
+            member->offset = current_unit_offset + (current_unit_bits_used / 8);
+            member->bit_offset = (uint8_t)(current_unit_bits_used % 8);
+            current_unit_bits_used += member->bit_width;
 
-            size_t aligned = _infix_align_up(current_byte_offset, member_align);
-            if (aligned < current_byte_offset) {
-                _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
-                return false;
-            }
-            current_byte_offset = aligned;
-            member->offset = current_byte_offset;
-
-            if (member_align > max_alignment)
-                max_alignment = member_align;
-            continue;  // FAM logic done
+            size_t bytes_used = (current_unit_bits_used + 7) / 8;
+            if (current_unit_offset + bytes_used > current_byte_offset)
+                current_byte_offset = current_unit_offset + bytes_used;
         }
-
-        // 2. Handle Bitfields
-        if (member->is_bitfield) {
-            // Zero-width bitfield: force alignment to the next boundary of the declared type.
-            if (member->bit_width == 0) {
-                if (current_bit_offset > 0) {
-                    if (current_byte_offset == SIZE_MAX) {
-                        _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
-                        return false;
-                    }
-                    current_byte_offset++;
-                    current_bit_offset = 0;
-                }
-                size_t align = member->type->alignment;
+        else {
+            // Handle Flexible Array Members (FAM)
+            if (mtype->category == INFIX_TYPE_ARRAY && mtype->meta.array_info.is_flexible) {
+                in_bitfield = false;
+                size_t align = mtype->alignment;
                 if (align == 0)
                     align = 1;
 
+                if (align > max_alignment && !type->meta.aggregate_info.is_packed)
+                    max_alignment = align;
+
                 size_t aligned = _infix_align_up(current_byte_offset, align);
                 if (aligned < current_byte_offset) {
                     _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
@@ -276,88 +293,36 @@ static bool _layout_struct(infix_type * type) {
                 current_byte_offset = aligned;
                 member->offset = current_byte_offset;
                 member->bit_offset = 0;
-
-                if (align > max_alignment)
-                    max_alignment = align;
                 continue;
             }
 
-            // Standard Bitfield
-            // Simplified System V packing: pack into current byte if it fits.
-            if (current_bit_offset + member->bit_width > 8) {
-                // Overflow: move to start of next byte
-                if (current_byte_offset == SIZE_MAX) {
-                    _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
-                    return false;
-                }
-                current_byte_offset++;
-                current_bit_offset = 0;
-            }
-
-            member->offset = current_byte_offset;
-            member->bit_offset = current_bit_offset;
-            current_bit_offset += member->bit_width;
-
-            // If we filled the byte exactly, advance to next byte
-            if (current_bit_offset == 8) {
-                if (current_byte_offset == SIZE_MAX) {
-                    _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
-                    return false;
-                }
-                current_byte_offset++;
-                current_bit_offset = 0;
-            }
-
-            // Update struct alignment. Bitfields typically impose the alignment of their base type.
-            size_t align = member->type->alignment;
+            // Standard Member
+            in_bitfield = false;
+            size_t align = mtype->alignment;
             if (align == 0)
                 align = 1;
-            if (align > max_alignment)
+
+            if (align > max_alignment && !type->meta.aggregate_info.is_packed)
                 max_alignment = align;
-        }
-        else {
-            // 3. Standard Member
 
-            // Flush bits first
-            if (current_bit_offset > 0) {
-                if (current_byte_offset == SIZE_MAX) {
-                    _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
-                    return false;
-                }
-                current_byte_offset++;
-                current_bit_offset = 0;
-            }
-
-            size_t member_align = member->type->alignment;
-            if (member_align == 0)
-                member_align = 1;
-
-            if (member_align > max_alignment)
-                max_alignment = member_align;
-
-            size_t aligned = _infix_align_up(current_byte_offset, member_align);
+            size_t aligned = _infix_align_up(current_byte_offset, align);
             if (aligned < current_byte_offset) {
                 _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
                 return false;
             }
             current_byte_offset = aligned;
             member->offset = current_byte_offset;
+            member->bit_offset = 0;
 
-            if (current_byte_offset > SIZE_MAX - member->type->size) {
+            if (current_byte_offset > SIZE_MAX - mtype->size) {
                 _infix_set_error(INFIX_CATEGORY_PARSER, INFIX_CODE_INTEGER_OVERFLOW, 0);
                 return false;
             }
-            current_byte_offset += member->type->size;
+            current_byte_offset += mtype->size;
         }
     }
 
-    // Final flush
-    if (current_bit_offset > 0)
-        current_byte_offset++;
-
-    // If it is packed, the alignment is explicitly determined by the user (defaulting to 1
-    // if not specified in the syntax). We must respect this value absolutely, ignoring
-    // the natural alignment of members.
+    // If it is packed, the alignment is explicitly determined by the user.
     if (type->meta.aggregate_info.is_packed)
         max_alignment = type->alignment;
 
@@ -428,8 +393,9 @@ static infix_status _create_aggregate_setup(infix_arena_t * arena,
 c23_nodiscard infix_status infix_type_create_pointer_to(infix_arena_t * arena,
                                                         infix_type ** out_type,
                                                         infix_type * pointee_type) {
+    _infix_clear_error();
     if (!out_type || !pointee_type) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     infix_type * type = infix_arena_calloc(arena, 1, sizeof(infix_type), _Alignof(infix_type));
@@ -455,12 +421,13 @@ c23_nodiscard infix_status infix_type_create_pointer_to(infix_arena_t * arena,
  * @param[in] num_elements The number of elements.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_array(infix_arena_t * arena,
-                                                   infix_type ** out_type,
-                                                   infix_type * element_type,
-                                                   size_t num_elements) {
+INFIX_API c23_nodiscard infix_status infix_type_create_array(infix_arena_t * arena,
+                                                             infix_type ** out_type,
+                                                             infix_type * element_type,
+                                                             size_t num_elements) {
+    _infix_clear_error();
     if (out_type == nullptr || element_type == nullptr) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     if (element_type->size > 0 && num_elements > SIZE_MAX / element_type->size) {
@@ -493,11 +460,12 @@ c23_nodiscard infix_status infix_type_create_array(infix_arena_t * arena,
  * @param[in] element_type The type of the array elements.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_flexible_array(infix_arena_t * arena,
-                                                            infix_type ** out_type,
-                                                            infix_type * element_type) {
+INFIX_API c23_nodiscard infix_status infix_type_create_flexible_array(infix_arena_t * arena,
+                                                                      infix_type ** out_type,
+                                                                      infix_type * element_type) {
+    _infix_clear_error();
     if (out_type == nullptr || element_type == nullptr) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     // Flexible arrays of incomplete types (size 0) are generally not allowed.
@@ -536,11 +504,12 @@ c23_nodiscard infix_status infix_type_create_flexible_array(infix_arena_t * aren
  * `infix_type_create_primitive(INFIX_PRIMITIVE_SINT32)`).
  * @return `INFIX_SUCCESS` on success, or `INFIX_ERROR_INVALID_ARGUMENT` if the underlying type is not an integer.
  */
-c23_nodiscard infix_status infix_type_create_enum(infix_arena_t * arena,
-                                                  infix_type ** out_type,
-                                                  infix_type * underlying_type) {
+INFIX_API c23_nodiscard infix_status infix_type_create_enum(infix_arena_t * arena,
+                                                            infix_type ** out_type,
+                                                            infix_type * underlying_type) {
+    _infix_clear_error();
     if (out_type == nullptr || underlying_type == nullptr) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     if (underlying_type->category != INFIX_TYPE_PRIMITIVE ||
@@ -570,11 +539,12 @@ c23_nodiscard infix_status infix_type_create_enum(infix_arena_t * arena,
  * @param[in] base_type The base floating-point type (`float` or `double`).
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_complex(infix_arena_t * arena,
-                                                     infix_type ** out_type,
-                                                     infix_type * base_type) {
+INFIX_API c23_nodiscard infix_status infix_type_create_complex(infix_arena_t * arena,
+                                                               infix_type ** out_type,
+                                                               infix_type * base_type) {
+    _infix_clear_error();
     if (out_type == nullptr || base_type == nullptr || (!is_float(base_type) && !is_double(base_type))) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     infix_type * type = infix_arena_calloc(arena, 1, sizeof(infix_type), _Alignof(infix_type));
@@ -600,12 +570,13 @@ c23_nodiscard infix_status infix_type_create_complex(infix_arena_t * arena,
  * @param[in] num_elements The number of elements in the vector.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_vector(infix_arena_t * arena,
-                                                    infix_type ** out_type,
-                                                    infix_type * element_type,
-                                                    size_t num_elements) {
+INFIX_API c23_nodiscard infix_status infix_type_create_vector(infix_arena_t * arena,
+                                                              infix_type ** out_type,
+                                                              infix_type * element_type,
+                                                              size_t num_elements) {
+    _infix_clear_error();
     if (out_type == nullptr || element_type == nullptr || element_type->category != INFIX_TYPE_PRIMITIVE) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     if (element_type->size > 0 && num_elements > SIZE_MAX / element_type->size) {
@@ -624,9 +595,9 @@ c23_nodiscard infix_status infix_type_create_vector(infix_arena_t * arena,
     type->meta.vector_info.element_type = element_type;
     type->meta.vector_info.num_elements = num_elements;
     type->size = element_type->size * num_elements;
-    // Vector alignment is typically its total size, up to a platform-specific maximum (e.g., 16 on x64).
-    // This is a simplification; the ABI-specific classifiers will handle the true alignment rules.
-    type->alignment = type->size > 8 ? 16 : type->size;
+    // Vector alignment is its total size.
+    // This ensures __m128 is 16-byte aligned, __m256 is 32-byte, and __m512 is 64-byte.
+    type->alignment = type->size;
     *out_type = type;
     return INFIX_SUCCESS;
 }
@@ -638,10 +609,11 @@ c23_nodiscard infix_status infix_type_create_vector(infix_arena_t * arena,
  * @param[in] num_members The number of members.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_union(infix_arena_t * arena,
-                                                   infix_type ** out_type,
-                                                   infix_struct_member * members,
-                                                   size_t num_members) {
+INFIX_API c23_nodiscard infix_status infix_type_create_union(infix_arena_t * arena,
+                                                             infix_type ** out_type,
+                                                             infix_struct_member * members,
+                                                             size_t num_members) {
+    _infix_clear_error();
     infix_type * type = nullptr;
     infix_struct_member * arena_members = nullptr;
     infix_status status = _create_aggregate_setup(arena, &type, &arena_members, members, num_members);
@@ -685,10 +657,10 @@ c23_nodiscard infix_status infix_type_create_union(infix_arena_t * arena,
  * @param[in] num_members The number of members in the array.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_struct(infix_arena_t * arena,
-                                                    infix_type ** out_type,
-                                                    infix_struct_member * members,
-                                                    size_t num_members) {
+INFIX_API c23_nodiscard infix_status infix_type_create_struct(infix_arena_t * arena,
+                                                              infix_type ** out_type,
+                                                              infix_struct_member * members,
+                                                              size_t num_members) {
     _infix_clear_error();
     infix_type * type = nullptr;
     infix_struct_member * arena_members = nullptr;
@@ -744,14 +716,24 @@ c23_nodiscard infix_status infix_type_create_struct(infix_arena_t * arena,
  * @param[in] num_members The number of members.
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_packed_struct(infix_arena_t * arena,
-                                                           infix_type ** out_type,
-                                                           size_t total_size,
-                                                           size_t alignment,
-                                                           infix_struct_member * members,
-                                                           size_t num_members) {
-    if (out_type == nullptr || (num_members > 0 && members == nullptr) || alignment == 0) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+INFIX_API c23_nodiscard infix_status infix_type_create_packed_struct(infix_arena_t * arena,
+                                                                     infix_type ** out_type,
+                                                                     size_t total_size,
+                                                                     size_t alignment,
+                                                                     infix_struct_member * members,
+                                                                     size_t num_members) {
+    _infix_clear_error();
+    if (out_type == nullptr || (num_members > 0 && members == nullptr)) {
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
+        return INFIX_ERROR_INVALID_ARGUMENT;
+    }
+    // Validate alignment is power-of-two AND within sane limits.
+    if (alignment == 0 || (alignment & (alignment - 1)) != 0) {
+        _infix_set_error(INFIX_CATEGORY_ALLOCATION, INFIX_CODE_INVALID_ALIGNMENT, 0);
+        return INFIX_ERROR_INVALID_ARGUMENT;
+    }
+    if (alignment > INFIX_MAX_ALIGNMENT) {
+        _infix_set_error(INFIX_CATEGORY_ABI, INFIX_CODE_TYPE_TOO_LARGE, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     infix_type * type = infix_arena_calloc(arena, 1, sizeof(infix_type), _Alignof(infix_type));
@@ -792,12 +774,13 @@ c23_nodiscard infix_status infix_type_create_packed_struct(infix_arena_t * arena
  * @param[in] agg_cat The expected category of the aggregate (struct or union).
  * @return `INFIX_SUCCESS` on success.
  */
-c23_nodiscard infix_status infix_type_create_named_reference(infix_arena_t * arena,
-                                                             infix_type ** out_type,
-                                                             const char * name,
-                                                             infix_aggregate_category_t agg_cat) {
+INFIX_API c23_nodiscard infix_status infix_type_create_named_reference(infix_arena_t * arena,
+                                                                       infix_type ** out_type,
+                                                                       const char * name,
+                                                                       infix_aggregate_category_t agg_cat) {
+    _infix_clear_error();
     if (out_type == nullptr || name == nullptr) {
-        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_UNKNOWN, 0);
+        _infix_set_error(INFIX_CATEGORY_GENERAL, INFIX_CODE_NULL_POINTER, 0);
         return INFIX_ERROR_INVALID_ARGUMENT;
     }
     infix_type * type = infix_arena_calloc(arena, 1, sizeof(infix_type), _Alignof(infix_type));
@@ -1234,7 +1217,7 @@ static size_t _estimate_graph_size_recursive(infix_arena_t * temp_arena,
 size_t _infix_estimate_graph_size(infix_arena_t * temp_arena, const infix_type * type) {
     if (!temp_arena || !type)
         return 0;
-    estimate_visited_node_t * visited_head = NULL;
+    estimate_visited_node_t * visited_head = nullptr;
     return _estimate_graph_size_recursive(temp_arena, type, &visited_head);
 }
 // Public API: Introspection Functions
@@ -1244,7 +1227,7 @@ size_t _infix_estimate_graph_size(infix_arena_t * temp_arena, const infix_type *
  * @return The name of the type if it was created from a registry alias (e.g., "MyInt"), or `nullptr` if the type is
  * anonymous.
  */
-c23_nodiscard const char * infix_type_get_name(const infix_type * type) {
+INFIX_API c23_nodiscard const char * infix_type_get_name(const infix_type * type) {
     if (type == nullptr)
         return nullptr;
     return type->name;
@@ -1254,7 +1237,7 @@ c23_nodiscard const char * infix_type_get_name(const infix_type * type) {
  * @param[in] type The type object to inspect.
  * @return The `infix_type_category` enum value, or -1 if `type` is `nullptr`.
  */
-c23_nodiscard infix_type_category infix_type_get_category(const infix_type * type) {
+INFIX_API c23_nodiscard infix_type_category infix_type_get_category(const infix_type * type) {
     return type ? type->category : (infix_type_category)-1;
 }
 /**
@@ -1262,20 +1245,20 @@ c23_nodiscard infix_type_category infix_type_get_category(const infix_type * typ
  * @param[in] type The type object to inspect.
  * @return The size in bytes, or 0 if `type` is `nullptr`.
  */
-c23_nodiscard size_t infix_type_get_size(const infix_type * type) { return type ? type->size : 0; }
+INFIX_API c23_nodiscard size_t infix_type_get_size(const infix_type * type) { return type ? type->size : 0; }
 /**
  * @brief Gets the alignment requirement of a type in bytes.
  * @param[in] type The type object to inspect.
  * @return The alignment in bytes, or 0 if `type` is `nullptr`.
  */
-c23_nodiscard size_t infix_type_get_alignment(const infix_type * type) { return type ? type->alignment : 0; }
+INFIX_API c23_nodiscard size_t infix_type_get_alignment(const infix_type * type) { return type ? type->alignment : 0; }
 /**
  * @brief Gets the number of members in a struct or union type.
  * @param[in] type The aggregate type object to inspect. Must have category
  * `INFIX_TYPE_STRUCT` or `INFIX_TYPE_UNION`.
  * @return The number of members, or 0 if the type is not a struct or union.
  */
-c23_nodiscard size_t infix_type_get_member_count(const infix_type * type) {
+INFIX_API c23_nodiscard size_t infix_type_get_member_count(const infix_type * type) {
     if (!type || (type->category != INFIX_TYPE_STRUCT && type->category != INFIX_TYPE_UNION))
         return 0;
     return type->meta.aggregate_info.num_members;
@@ -1286,7 +1269,7 @@ c23_nodiscard size_t infix_type_get_member_count(const infix_type * type) {
  * @param[in] index The zero-based index of the member.
  * @return A pointer to the `infix_struct_member`, or `nullptr` if the index is out of bounds or the type is invalid.
  */
-c23_nodiscard const infix_struct_member * infix_type_get_member(const infix_type * type, size_t index) {
+INFIX_API c23_nodiscard const infix_struct_member * infix_type_get_member(const infix_type * type, size_t index) {
     if (!type || (type->category != INFIX_TYPE_STRUCT && type->category != INFIX_TYPE_UNION) ||
         index >= type->meta.aggregate_info.num_members)
         return nullptr;
@@ -1299,7 +1282,7 @@ c23_nodiscard const infix_struct_member * infix_type_get_member(const infix_type
  * @return The name of the argument as a string, or `nullptr` if the argument is anonymous or the index is out of
  * bounds.
  */
-c23_nodiscard const char * infix_type_get_arg_name(const infix_type * func_type, size_t index) {
+INFIX_API c23_nodiscard const char * infix_type_get_arg_name(const infix_type * func_type, size_t index) {
     if (!func_type || func_type->category != INFIX_TYPE_REVERSE_TRAMPOLINE ||
         index >= func_type->meta.func_ptr_info.num_args)
         return nullptr;
@@ -1311,7 +1294,7 @@ c23_nodiscard const char * infix_type_get_arg_name(const infix_type * func_type,
  * @param[in] index The zero-based index of the argument.
  * @return A pointer to the argument's `infix_type`, or `nullptr` if the index is out of bounds.
  */
-c23_nodiscard const infix_type * infix_type_get_arg_type(const infix_type * func_type, size_t index) {
+INFIX_API c23_nodiscard const infix_type * infix_type_get_arg_type(const infix_type * func_type, size_t index) {
     if (!func_type || func_type->category != INFIX_TYPE_REVERSE_TRAMPOLINE ||
         index >= func_type->meta.func_ptr_info.num_args)
         return nullptr;
@@ -1322,7 +1305,7 @@ c23_nodiscard const infix_type * infix_type_get_arg_type(const infix_type * func
  * @param[in] trampoline The trampoline handle.
  * @return The number of arguments, or 0 if `trampoline` is `nullptr`.
  */
-c23_nodiscard size_t infix_forward_get_num_args(const infix_forward_t * trampoline) {
+INFIX_API c23_nodiscard size_t infix_forward_get_num_args(const infix_forward_t * trampoline) {
     return trampoline ? trampoline->num_args : 0;
 }
 /**
@@ -1330,7 +1313,7 @@ c23_nodiscard size_t infix_forward_get_num_args(const infix_forward_t * trampoli
  * @param[in] trampoline The trampoline handle.
  * @return The number of fixed arguments, or 0 if `trampoline` is `nullptr`.
  */
-c23_nodiscard size_t infix_forward_get_num_fixed_args(const infix_forward_t * trampoline) {
+INFIX_API c23_nodiscard size_t infix_forward_get_num_fixed_args(const infix_forward_t * trampoline) {
     return trampoline ? trampoline->num_fixed_args : 0;
 }
 /**
@@ -1338,7 +1321,7 @@ c23_nodiscard size_t infix_forward_get_num_fixed_args(const infix_forward_t * tr
  * @param[in] trampoline The trampoline handle.
  * @return A pointer to the `infix_type` for the return value, or `nullptr` if `trampoline` is `nullptr`.
  */
-c23_nodiscard const infix_type * infix_forward_get_return_type(const infix_forward_t * trampoline) {
+INFIX_API c23_nodiscard const infix_type * infix_forward_get_return_type(const infix_forward_t * trampoline) {
     return trampoline ? trampoline->return_type : nullptr;
 }
 /**
@@ -1347,7 +1330,8 @@ c23_nodiscard const infix_type * infix_forward_get_return_type(const infix_forwa
  * @param[in] index The zero-based index of the argument.
  * @return A pointer to the `infix_type`, or `nullptr` if the index is out of bounds or `trampoline` is `nullptr`.
  */
-c23_nodiscard const infix_type * infix_forward_get_arg_type(const infix_forward_t * trampoline, size_t index) {
+INFIX_API c23_nodiscard const infix_type * infix_forward_get_arg_type(const infix_forward_t * trampoline,
+                                                                      size_t index) {
     if (!trampoline || index >= trampoline->num_args)
         return nullptr;
     return trampoline->arg_types[index];
@@ -1357,7 +1341,7 @@ c23_nodiscard const infix_type * infix_forward_get_arg_type(const infix_forward_
  * @param[in] trampoline The trampoline context handle.
  * @return The number of arguments, or 0 if `trampoline` is `nullptr`.
  */
-c23_nodiscard size_t infix_reverse_get_num_args(const infix_reverse_t * trampoline) {
+INFIX_API c23_nodiscard size_t infix_reverse_get_num_args(const infix_reverse_t * trampoline) {
     return trampoline ? trampoline->num_args : 0;
 }
 /**
@@ -1365,7 +1349,7 @@ c23_nodiscard size_t infix_reverse_get_num_args(const infix_reverse_t * trampoli
  * @param[in] trampoline The trampoline context handle.
  * @return The number of fixed arguments, or 0 if `trampoline` is `nullptr`.
  */
-c23_nodiscard size_t infix_reverse_get_num_fixed_args(const infix_reverse_t * trampoline) {
+INFIX_API c23_nodiscard size_t infix_reverse_get_num_fixed_args(const infix_reverse_t * trampoline) {
     return trampoline ? trampoline->num_fixed_args : 0;
 }
 /**
@@ -1373,7 +1357,7 @@ c23_nodiscard size_t infix_reverse_get_num_fixed_args(const infix_reverse_t * tr
  * @param[in] trampoline The trampoline context handle.
  * @return A pointer to the `infix_type` for the return value, or `nullptr` if `trampoline` is `nullptr`.
  */
-c23_nodiscard const infix_type * infix_reverse_get_return_type(const infix_reverse_t * trampoline) {
+INFIX_API c23_nodiscard const infix_type * infix_reverse_get_return_type(const infix_reverse_t * trampoline) {
     return trampoline ? trampoline->return_type : nullptr;
 }
 /**
@@ -1382,7 +1366,8 @@ c23_nodiscard const infix_type * infix_reverse_get_return_type(const infix_rever
  * @param[in] index The zero-based index of the argument.
  * @return A pointer to the `infix_type`, or `nullptr` if the index is out of bounds or `trampoline` is `nullptr`.
  */
-c23_nodiscard const infix_type * infix_reverse_get_arg_type(const infix_reverse_t * trampoline, size_t index) {
+INFIX_API c23_nodiscard const infix_type * infix_reverse_get_arg_type(const infix_reverse_t * trampoline,
+                                                                      size_t index) {
     if (!trampoline || index >= trampoline->num_args)
         return nullptr;
     return trampoline->arg_types[index];
